/* Hawaiisim definitions (c.f ircam/hawaii_pbsrc/hawaii.h). */
#define AUTHOR "Richard Neill"
#define YEAR "2011-2013"
#define LICENSE "GPL3+"

/* Hawaii physical layout */
#define NUM_ROWS 	512
#define NUM_COLS 	512
#define QUADS	 	4
#define RC_BEFORE_START	-1	/* Row or col register is reset to before the start of the array.  */
#define RC_UNINIT	-100	/* uninitialised. */

/* ADC is 24 bits */
#define MAX_24BIT 	0xffffff

/* Pulseblaster lines, as we have them connected */
#define PB_BITS 	24	/* 16 bits in bus, + the high-bit for Fake ADC */

#define PIXEL_0_BIT	0	/* Individual bit numbers */
#define PIXEL_1_BIT	1
#define PIXEL_2_BIT	2
#define PIXEL_3_BIT	3

#define LINE_0_BIT  	4
#define LINE_1_BIT	5
#define LINE_2_BIT	6
#define LINE_3_BIT	7

#define READ_BIT    	8
#define RESETB_BIT  	12
#define LSYNC_BIT   	13
#define FSYNC_BIT   	14
#define ADC_BIT		15
#define FAKEADC_BIT	23

#define PIXEL_0		0x1 << PIXEL_0_BIT	/* words, with this bit set. More useful */
#define PIXEL_1		0x1 << PIXEL_1_BIT
#define PIXEL_2		0x1 << PIXEL_2_BIT
#define PIXEL_3		0x1 << PIXEL_3_BIT

#define LINE_0  	0x1 << LINE_0_BIT
#define LINE_1  	0x1 << LINE_1_BIT
#define LINE_2		0x1 << LINE_2_BIT
#define LINE_3  	0x1 << LINE_3_BIT

#define READ    	0x1 << READ_BIT
#define RESETB  	0x1 << RESETB_BIT
#define LSYNC   	0x1 << LSYNC_BIT
#define FSYNC   	0x1 << FSYNC_BIT
#define ADC		0x1 << ADC_BIT
#define FAKEADC		0x1 << FAKEADC_BIT

#define PIXELS  	(PIXEL_0 | PIXEL_1 | PIXEL_2 | PIXEL_3)		/* grouped bits */
#define LINES   	(LINE_0 | LINE_1 | LINE_2 | LINE_3)
#define POSN_CLOCKS	(PIXELS|LINES|LSYNC|FSYNC)			/* the lines used for positioning */
#define USED    	(PIXELS|LINES|LSYNC|FSYNC|READ|RESETB|ADC|FAKEADC)	/* all the lines that are actually in use */

#define INIT_SANE (ADC)		/* The sane first word we can send, after any random powerup state...this won't upset any rules */

/* Pulse requirements */
#define MIN_PULSE_POSN_NS 	100	/* shortest allowed pulse (either phase) for one of the position clocks (ns) */
#define MIN_PULSE_RESET_NS	500   	/* shortest allowed pulse (either phase) on the reset line (ns) */
#define MIN_PULSE_ADC_NS	50	/* shortest allowed pulse (either phase) on the ADC line (ns) */
#define MIN_PULSE_FAKEADC_NS	2500	/* shortest allowed pulse (either phase) on the ADC line (ns) */
#define MIN_INTERVAL_FAKEADC_NS	  5000	/* shortest allowed interval between (fake) ADC triggers (falling edge) (ns) */
#define MIN_SETUP_READ_FAKEADC_NS 2500	/* shortest allowed setup interval on the READ line (between read high and ADC fe)( (ns) */
#define MIN_HOLD_READ_FAKEADC_NS  2500	/* shortest allowed hold time on the READ line (between ADC trigger and READ going low) (ns) */

/* Signal and Noise levels (defaults) */
#define RESET_LEVEL		1000	/* The "target" reset level to which the cells discharge. */
#define RESET_TCONST_US		0.5	/* The time-constant of the reset circuit, in us. Would ideally be zero if resetting were perfect. */

#define GLOW_EUS		0.015	/* The average number of "glow" electrons/us. Fullscale in 1130 seconds. */
#define LIGHT_EUS		1.60	/* The exposure rate to "uniform" light. Average electrons/us. Fullscale in 10 seconds. */
#define TIFF_EUS		1.60	/* The sensitivity for the tiff: electrons/us for a white pixel */
#define BACKGROUND_EUS		0.02	/* The average amount of background 'noise': stray light + glow from elsewhere than source-follower. */

#define HAWAII_C_NOISE		250.0	/* The hawaii thermal noise, at reset. (mainly kTC capacitor noise) */
#define ADC_R_NOISE		400.0	/* The noise per ADC sample operation (mainly resistor Johnson noise) */

#define MAINS_FREQ_HZ		50	/* Mains 'hum'. Assume 50 Hz, perfect sine. Real world is within 0.2 Hz, usually much better */
#define MAINS_HUM_AMPLITUDE	500    	/* Amplitude of induced hum. Should be nearly zero with triax-cable and no earth loops! (~100k produces visible artefacts.) */

/* Gencam simulation (defaults) */
#define GENCAM_RESET_US		1.5	/* reset length (us) in gen_cam mode (should be ~ 1.5us, i.e. 3 x time-const) */
#define GENCAM_EXPOSURE_US	7500000 /* exposure length (us) in gen_cam mode (should be ~< 10s). (7.5 s is good: allow some headroom for noise/glow, esp. with white tiff, also time for full-frame ADC to happen. */

/* Reset, Exposure, and Read modes (effectively an enum) */
#define RESET_MODE_ZERO   	0	/* Reset mode: reset to zero */
#define RESET_MODE_CONST 	1 	/* Reset mode: reset to constant */
#define RESET_MODE_NOISE 	2	/* Reset mode: reset to constant + noise */
#define RESET_MODE_EXPON  	3	/* Reset mode: decay towards constant + noise */

#define EXPOSURE_MODE_DEBUG   	0	/* Exposure mode: debugging, own address */
#define EXPOSURE_MODE_LINEAR  	1	/* Exposure mode: linear */
#define EXPOSURE_MODE_GLOW   	2	/* Exposure mode: linear with glow */
#define EXPOSURE_MODE_EQUALTIFF 3	/* Exposure mode: set pixel to tiff values, no integration or noise (used for speed) */
#define EXPOSURE_MODE_LINEARTIFF 4	/* Exposure mode: linear integration from image, without glow */
#define EXPOSURE_MODE_IMAGETIFF 5	/* Exposure mode: data integration from image, with glow */

#define ADC_MODE_IDEAL  	0	/* ADC mode: read exactly */
#define ADC_MODE_NOISE		1	/* ADC mode: read with ADC noise */
#define ADC_MODE_HUM		2	/* ADC mode: read with ADC noise and mains hum */

/* File extensions */
#define EXTN_PBSIM 	"pbsim"
#define EXTN_CAM   	"cam"
#define EXTN_DAT   	"dat"
#define EXTN_TIFF   	"tiff"

/* Misc */
#define FILENAME_LEN 	255
#define BUFFER_LEN   	1024
#define NS_MS		1000000		/* ns per ms */
#define NS_US		1000		/* ns per us */
#define NS_S		1000000000	/* ns per s  */
#define ERROR_RED	"\033[1;31mError\033[0m"  /* 'Error', in red */
#define RESET_GREEN	"\033[1;32mReset\033[0m"  /* 'Reset', in green */
#define READ_BLUE	"\033[1;34mRead\033[0m"   /* 'Read', in blue */
#define NITRIG_YELLOW	"\033[1;33mNITrig\033[0m" /* 'NITrig', in yellow */
#define MAGENTA		"\033[1;35m"		  /*  For other highlighting */ 
#define NORM		"\033[0m"
#define DEV_NULL	"/dev/null"

